"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var path = require("path");
require("colors");
var makeResolver = require("./resolver");
var utils_1 = require("./utils");
function compile(fileNames, options, oldProgram) {
    var dependencyGraph = {};
    var errors = [];
    var reverseDependencyGraph = {};
    var host = createCompilerHost();
    // make a (sync) resolver that follows webpack's rules
    var resolveSync = makeResolver({ resolve: {} });
    var program = ts.createProgram(fileNames, options, host, oldProgram);
    var emitResult = program.emit();
    var allDiagnostics = ts.getPreEmitDiagnostics(program).concat(emitResult.diagnostics);
    allDiagnostics.forEach(function (diagnostic) {
        var errorLocation = { line: null, character: null };
        var rawMessage = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
        var errorMessage = rawMessage;
        if (diagnostic.file) {
            errorLocation = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
            errorMessage = "" + '('.white + (errorLocation.line + 1).toString().cyan + "," + (errorLocation.character + 1).toString().cyan + "): " + rawMessage.red;
            errorMessage = path.normalize(diagnostic.file.fileName).red + errorMessage;
        }
        var fileName = diagnostic.file ? diagnostic.file.fileName : '--------';
        errors.push({
            file: fileName,
            location: errorLocation,
            rawMessage: rawMessage,
            message: errorMessage,
            loaderSource: 'ts-loader'
        });
    });
    return { program: program, dependencyGraph: dependencyGraph, errors: errors };
    function createCompilerHost() {
        var compilerHost = ts.createCompilerHost(options);
        compilerHost.resolveModuleNames = resolveModuleNames;
        return compilerHost;
        function resolveModuleNames(moduleNames, containingFile) {
            var resolvedModules = moduleNames.map(function (moduleName) {
                return resolveModuleName(resolveSync, [], /\.tsx?$/i, moduleName, containingFile);
            });
            populateDependencyGraphs(resolvedModules, containingFile);
            return resolvedModules;
        }
        function resolveModuleName(resolveSync, appendTsSuffixTo, scriptRegex, moduleName, containingFile) {
            var resolutionResult;
            try {
                var resolvedFileName = resolveSync(undefined, path.normalize(path.dirname(containingFile)), moduleName);
                resolvedFileName = utils_1.appendTsSuffixIfMatch(appendTsSuffixTo, resolvedFileName);
                if (resolvedFileName.match(scriptRegex)) {
                    resolutionResult = { resolvedFileName: resolvedFileName };
                }
            }
            catch (e) { }
            var moduleResolutionHost = {
                fileExists: compilerHost.fileExists,
                readFile: compilerHost.readFile
            };
            var tsResolution = ts.resolveModuleName(moduleName, containingFile, options, moduleResolutionHost);
            if (tsResolution.resolvedModule) {
                var tsResolutionResult = {
                    resolvedFileName: path.normalize(tsResolution.resolvedModule.resolvedFileName),
                    isExternalLibraryImport: tsResolution.resolvedModule.isExternalLibraryImport
                };
                if (resolutionResult) {
                    if (resolutionResult.resolvedFileName === tsResolutionResult.resolvedFileName) {
                        resolutionResult.isExternalLibraryImport = tsResolutionResult.isExternalLibraryImport;
                    }
                }
                else {
                    resolutionResult = tsResolutionResult;
                }
            }
            return resolutionResult;
        }
        function populateDependencyGraphs(resolvedModules, containingFile) {
            var importedFiles = resolvedModules
                .filter(function (m) { return m !== null && m !== undefined; })
                .map(function (m) { return m.resolvedFileName; });
            dependencyGraph[path.normalize(containingFile)] = importedFiles;
            importedFiles.forEach(function (importedFileName) {
                if (!reverseDependencyGraph[importedFileName]) {
                    reverseDependencyGraph[importedFileName] = {};
                }
                reverseDependencyGraph[importedFileName][path.normalize(containingFile)] = true;
            });
        }
    }
}
exports.compile = compile;
//# sourceMappingURL=compiler.js.map