"use strict";
var ts = require("typescript");
var path = require("path");
var fs = require("fs");
var logger_1 = require("./logger");
var compiler_1 = require("./compiler");
var utils_1 = require("./utils");
var TSPlugin = (function () {
    function TSPlugin(options) {
        this.options = options;
        this.options.cacheDir = options.cacheDir || '.typescript-webpack-plugin';
        //START overriding tsconfig
        this.options.tsconfig.compilerOptions.outDir = path.join(this.options.cacheDir, 'build');
        this.options.tsconfig.compilerOptions.rootDir = process.cwd();
        //END overriding tsconfig
        this.tsOutputDir = path.resolve(options.tsconfig.compilerOptions.outDir);
        this.cachePath = path.join(this.options.cacheDir, 'cache.json');
    }
    TSPlugin.prototype.apply = function (compiler) {
        var _this = this;
        var oldProgram;
        // will be passed to loader in order to add dependencies
        var tsDependencyGraph;
        var compilationResult;
        var allSources = this.getAllSources();
        compiler.plugin("compile", function () {
            logger_1.logger.info("starting typescript@" + ts.version + " commpilation");
            logger_1.logger.profile('compilation');
            var allSourceHashes = utils_1.calculateSourcesCheckSums(allSources);
            var cachedResults = _this.readCachedResults();
            var sourcesToCompile = utils_1.getChangedSourcesAndDepenencies(allSources, allSourceHashes, cachedResults);
            logger_1.logger.info("starting to compile [%s] files, [%s] cached, reusing program: ", sourcesToCompile.length, allSources.length - sourcesToCompile.length, !!oldProgram);
            compilationResult = compiler_1.compile(sourcesToCompile, _this.compilerOptions, oldProgram);
            oldProgram = compilationResult.program;
            cachedResults = _this.updateCachedResults(sourcesToCompile, compilationResult.dependencyGraph, allSourceHashes, cachedResults);
            tsDependencyGraph = cachedResults.dependencyGraph;
            logger_1.logger.profile('compilation');
        });
        compiler.plugin("compilation", function (compilation) {
            if (compilationResult && compilationResult.errors && compilationResult.errors.length > 0) {
                (_a = compilation.errors).push.apply(_a, compilationResult.errors);
                logger_1.logger.error("compilation fail: ", compilationResult.errors.map(function (error) { return error.message; }));
            }
            compilation.plugin('normal-module-loader', function (loaderContext) {
                loaderContext.tsPluginOptions = _this.options;
                loaderContext.tsOutputDir = _this.tsOutputDir;
                loaderContext.tsDependencyGraph = tsDependencyGraph;
            });
            var _a;
        });
    };
    ;
    TSPlugin.prototype.getAllSources = function () {
        var allSources = utils_1.getFilesGlob(this.options.tsconfig.filesGlob);
        allSources.push.apply(allSources, utils_1.getFilesGlob(this.options.include));
        var excluded = utils_1.getFilesGlob(this.options.exclude);
        return allSources.filter(function (file) { return excluded.indexOf(file) === -1; });
    };
    Object.defineProperty(TSPlugin.prototype, "compilerOptions", {
        get: function () {
            var _a = ts.convertCompilerOptionsFromJson(this.options.tsconfig.compilerOptions, '.'), options = _a.options, errors = _a.errors;
            if (errors && errors.length > 0)
                throw errors;
            return options;
        },
        enumerable: true,
        configurable: true
    });
    TSPlugin.prototype.readCachedResults = function () {
        var cachedResults = { checkSums: {}, dependencyGraph: {} };
        try {
            cachedResults = JSON.parse(fs.readFileSync(this.cachePath, 'utf-8'));
            cachedResults.dependencyGraph = cachedResults.dependencyGraph || {};
        }
        catch (error) {
            logger_1.logger.debug('cache is empty');
        }
        return cachedResults;
    };
    TSPlugin.prototype.updateCachedResults = function (sourcesToCompile, dependencyGraph, allSourceHashes, cachedResults) {
        sourcesToCompile.forEach(function (srcPath) { return cachedResults.checkSums[srcPath] = allSourceHashes[srcPath]; });
        Object.assign(cachedResults.dependencyGraph, dependencyGraph);
        try {
            if (!fs.existsSync(path.dirname(this.cachePath))) {
                fs.mkdirSync(path.dirname(this.cachePath));
            }
            fs.writeFileSync(this.cachePath, JSON.stringify(cachedResults), 'utf-8');
        }
        catch (error) {
            throw error;
        }
        return cachedResults;
    };
    return TSPlugin;
}());
module.exports = TSPlugin;
//# sourceMappingURL=plugin.js.map