"use strict";
var ts = require("typescript");
var glob = require("glob");
var path = require("path");
var fs = require("fs");
var compiler_1 = require("./compiler");
var utils_1 = require("./utils");
var TSPlugin = (function () {
    function TSPlugin(options) {
        this.options = options;
        this.options.cacheDir = options.cacheDir || '.typescript-webpack-plugin';
        this.options.tsconfig.compilerOptions.outDir = path.join(this.options.cacheDir, 'build');
        this.tsOutputDir = path.resolve(options.tsconfig.compilerOptions.outDir);
        this.cachePath = path.join(this.options.cacheDir, 'cache.json');
    }
    TSPlugin.prototype.apply = function (compiler) {
        var _this = this;
        var oldProgram;
        // will be passed to loader in order to add dependencies
        var tsDependencyGraph;
        var allSources = this.options.tsconfig.filesGlob
            .map(function (entry) { return glob.sync(entry); })
            .reduce(function (a1, a2) { return a1.concat(a2); });
        compiler.plugin("compile", function () {
            var allSourceHashes = utils_1.calculateSourcesCheckSums(allSources);
            var cachedResults = _this.readCachedResults();
            var sourcesToCompile = utils_1.getChangedSourcesAndDepenencies(allSources, allSourceHashes, cachedResults);
            console.log("starting to compile [%s] files, [%s] cached, reusing program: ", sourcesToCompile.length, allSources.length - sourcesToCompile.length, !!oldProgram);
            var startTime = Date.now();
            var _a = compiler_1.compile(sourcesToCompile, _this.compilerOptions, oldProgram), program = _a.program, dependencyGraph = _a.dependencyGraph;
            oldProgram = program;
            cachedResults = _this.updateCachedResults(sourcesToCompile, dependencyGraph, allSourceHashes, cachedResults);
            tsDependencyGraph = cachedResults.dependencyGraph;
            console.log("compilation finished: [%s] s.", (Date.now() - startTime) / 1000);
        });
        compiler.plugin("compilation", function (compilation) {
            compilation.plugin('normal-module-loader', function (loaderContext) {
                loaderContext.tsPluginOptions = _this.options;
                loaderContext.tsOutputDir = _this.tsOutputDir;
                loaderContext.tsDependencyGraph = tsDependencyGraph;
            });
        });
    };
    ;
    Object.defineProperty(TSPlugin.prototype, "compilerOptions", {
        get: function () {
            var _a = ts.convertCompilerOptionsFromJson(this.options.tsconfig.compilerOptions, '.'), options = _a.options, errors = _a.errors;
            if (errors && errors.length > 0)
                throw errors;
            return options;
        },
        enumerable: true,
        configurable: true
    });
    TSPlugin.prototype.readCachedResults = function () {
        var cachedResults = { checkSums: {}, dependencyGraph: {} };
        try {
            cachedResults = JSON.parse(fs.readFileSync(this.cachePath, 'utf-8'));
            cachedResults.dependencyGraph = cachedResults.dependencyGraph || {};
        }
        catch (error) {
            console.log('cache is empty');
        }
        return cachedResults;
    };
    TSPlugin.prototype.updateCachedResults = function (sourcesToCompile, dependencyGraph, allSourceHashes, cachedResults) {
        sourcesToCompile.forEach(function (srcPath) { return cachedResults.checkSums[srcPath] = allSourceHashes[srcPath]; });
        Object.assign(cachedResults.dependencyGraph, dependencyGraph);
        try {
            fs.writeFileSync(this.cachePath, JSON.stringify(cachedResults), 'utf-8');
        }
        catch (error) {
            throw error;
        }
        return cachedResults;
    };
    return TSPlugin;
}());
module.exports = TSPlugin;
//# sourceMappingURL=ts-plugin.js.map